@@ -28,6 +28,7 @@ from tenacity import (
     retry,
     stop_after_attempt,
     wait_exponential,
+    wait_fixed,
     retry_if_exception_type,
     after_log,
     wait_random,
@@ -115,9 +116,17 @@ class BaseTest(TestCase):
         self.output_file_path = None
         self.sub_input_file_path = None
 
+    @retry(
+        stop=stop_after_attempt(5),
+        wait=wait_fixed(30),
+        retry=retry_if_exception_type(Exception),
+    )
     def tearDown(self):
         if self.stack_name:
-            self.client_provider.cfn_client.delete_stack(StackName=self.stack_name)
+            client = self.client_provider.cfn_client
+            client.delete_stack(StackName=self.stack_name)
+            waiter = client.get_waiter("stack_delete_complete")
+            waiter.wait(StackName=self.stack_name)
         if self.output_file_path and os.path.exists(self.output_file_path):
             os.remove(self.output_file_path)
         if self.sub_input_file_path and os.path.exists(self.sub_input_file_path):
@@ -560,6 +569,17 @@ class BaseTest(TestCase):
             )
         return response
 
+    def verify_post_request(self, url: str, body_obj, expected_status_code: int):
+        """Return response to POST request and verify matches expected status code."""
+        response = self.do_post_request(url, body_obj)
+        if response.status_code != expected_status_code:
+            raise StatusCodeError(
+                "Request to {} failed with status: {}, expected status: {}".format(
+                    url, response.status_code, expected_status_code
+                )
+            )
+        return response
+
     def get_default_test_template_parameters(self):
         """
         get the default template parameters
@@ -633,3 +653,13 @@ class BaseTest(TestCase):
                 extra={"test": self.testcase, "status": response.status_code, "headers": amazon_headers},
             )
         return response
+
+    def do_post_request(self, url: str, body_obj):
+        """Perform a POST request with dict body body_obj."""
+        response = requests.post(url, json=body_obj)
+        if self.internal:
+            REQUEST_LOGGER.info(
+                "POST request made to " + url,
+                extra={"test": self.testcase, "status": response.status_code},
+            )
+        return response
