@@ -1,5 +1,6 @@
 import copy
 import re
+from typing import Any, Dict, Optional
 
 from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model import ResourceMacro, PropertyType
@@ -23,6 +24,7 @@ from samtranslator.model.exceptions import InvalidEventException, InvalidResourc
 from samtranslator.swagger.swagger import SwaggerEditor
 from samtranslator.open_api.open_api import OpenApiEditor
 from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
+from samtranslator.validator.value_validator import sam_expect
 
 CONDITION = "Condition"
 
@@ -54,6 +56,7 @@ class PushEventSource(ResourceMacro):
     # line to avoid any potential behavior change.
     # TODO: Make `PushEventSource` an abstract class and not giving `principal` initial value.
     principal: str = None  # type: ignore
+    relative_id: str  # overriding the Optional[str]: for event, relative id is not None
 
     def _construct_permission(  # type: ignore[no-untyped-def]
         self, function, source_arn=None, source_account=None, suffix="", event_source_token=None, prefix=None
@@ -69,8 +72,8 @@ class PushEventSource(ResourceMacro):
         if suffix.isalnum():
             permission_logical_id = prefix + "Permission" + suffix
         else:
-            generator = logical_id_generator.LogicalIdGenerator(prefix + "Permission", suffix)  # type: ignore[no-untyped-call]
-            permission_logical_id = generator.gen()  # type: ignore[no-untyped-call]
+            generator = logical_id_generator.LogicalIdGenerator(prefix + "Permission", suffix)
+            permission_logical_id = generator.gen()
         lambda_permission = LambdaPermission(
             permission_logical_id, attributes=function.get_passthrough_resource_attributes()
         )
@@ -107,7 +110,7 @@ class Schedule(PushEventSource):
         "RetryPolicy": PropertyType(False, is_type(dict)),
     }
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         """Returns the EventBridge Rule and Lambda Permission to which this Schedule event source corresponds.
 
@@ -140,7 +143,7 @@ class Schedule(PushEventSource):
         events_rule.Name = self.Name  # type: ignore[attr-defined]
         events_rule.Description = self.Description  # type: ignore[attr-defined]
 
-        source_arn = events_rule.get_runtime_attr("arn")  # type: ignore[no-untyped-call]
+        source_arn = events_rule.get_runtime_attr("arn")
         dlq_queue_arn = None
         if self.DeadLetterConfig is not None:  # type: ignore[attr-defined]
             EventBridgeRuleUtils.validate_dlq_config(self.logical_id, self.DeadLetterConfig)  # type: ignore[attr-defined, no-untyped-call]
@@ -192,7 +195,7 @@ class CloudWatchEvent(PushEventSource):
         "State": PropertyType(False, is_str()),
     }
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         """Returns the CloudWatch Events/EventBridge Rule and Lambda Permission to which
         this CloudWatch Events/EventBridge event source corresponds.
@@ -213,7 +216,7 @@ class CloudWatchEvent(PushEventSource):
         events_rule.EventBusName = self.EventBusName  # type: ignore[attr-defined]
         events_rule.EventPattern = self.Pattern  # type: ignore[attr-defined]
         events_rule.Name = self.RuleName  # type: ignore[attr-defined]
-        source_arn = events_rule.get_runtime_attr("arn")  # type: ignore[no-untyped-call]
+        source_arn = events_rule.get_runtime_attr("arn")
 
         dlq_queue_arn = None
         if self.DeadLetterConfig is not None:  # type: ignore[attr-defined]
@@ -288,7 +291,7 @@ class S3(PushEventSource):
                 return {"bucket": resources[bucket_id], "bucket_id": bucket_id}
         raise InvalidEventException(self.relative_id, "S3 events must reference an S3 bucket in the same template.")
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         """Returns the Lambda Permission resource allowing S3 to invoke the function this event source triggers.
 
@@ -392,7 +395,7 @@ class S3(PushEventSource):
                 "Fn::If": [permission.resource_attributes[CONDITION], ref(permission.logical_id), "no dependency"]
             }
         }
-        properties["Tags"] = tags + get_tag_list(dep_tag)  # type: ignore[no-untyped-call]
+        properties["Tags"] = tags + get_tag_list(dep_tag)
         return bucket
 
     def _inject_notification_configuration(self, function, bucket, bucket_id):  # type: ignore[no-untyped-def]
@@ -424,8 +427,7 @@ class S3(PushEventSource):
             notification_config = {}
             properties["NotificationConfiguration"] = notification_config
 
-        if not isinstance(notification_config, dict):
-            raise InvalidResourceException(bucket_id, "Invalid type for NotificationConfiguration.")
+        sam_expect(notification_config, bucket_id, "NotificationConfiguration").to_be_a_map()
 
         lambda_notifications = notification_config.get("LambdaConfigurations", None)
         if lambda_notifications is None:
@@ -451,9 +453,16 @@ class SNS(PushEventSource):
         "Region": PropertyType(False, is_str()),
         "FilterPolicy": PropertyType(False, dict_of(is_str(), list_of(one_of(is_str(), is_type(dict))))),
         "SqsSubscription": PropertyType(False, one_of(is_type(bool), is_type(dict))),
+        "RedrivePolicy": PropertyType(False, is_type(dict)),
     }
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    Topic: str
+    Region: Optional[str]
+    FilterPolicy: Optional[Dict[str, Any]]
+    SqsSubscription: Optional[Any]
+    RedrivePolicy: Optional[Dict[str, Any]]
+
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         """Returns the Lambda Permission resource allowing SNS to invoke the function this event source triggers.
 
@@ -468,27 +477,28 @@ class SNS(PushEventSource):
             raise TypeError("Missing required keyword argument: function")
 
         # SNS -> Lambda
-        if not self.SqsSubscription:  # type: ignore[attr-defined]
-            subscription = self._inject_subscription(  # type: ignore[no-untyped-call]
+        if not self.SqsSubscription:
+            subscription = self._inject_subscription(
                 "lambda",
                 function.get_runtime_attr("arn"),
-                self.Topic,  # type: ignore[attr-defined]
-                self.Region,  # type: ignore[attr-defined]
-                self.FilterPolicy,  # type: ignore[attr-defined]
+                self.Topic,
+                self.Region,
+                self.FilterPolicy,
+                self.RedrivePolicy,
                 function,
             )
-            return [self._construct_permission(function, source_arn=self.Topic), subscription]  # type: ignore[attr-defined, no-untyped-call]
+            return [self._construct_permission(function, source_arn=self.Topic), subscription]  # type: ignore[no-untyped-call]
 
         # SNS -> SQS(Create New) -> Lambda
-        if isinstance(self.SqsSubscription, bool):  # type: ignore[attr-defined]
+        if isinstance(self.SqsSubscription, bool):
             resources = []  # type: ignore[var-annotated]
             queue = self._inject_sqs_queue(function)  # type: ignore[no-untyped-call]
             queue_arn = queue.get_runtime_attr("arn")
             queue_url = queue.get_runtime_attr("queue_url")
 
-            queue_policy = self._inject_sqs_queue_policy(self.Topic, queue_arn, queue_url, function)  # type: ignore[attr-defined, no-untyped-call]
-            subscription = self._inject_subscription(  # type: ignore[no-untyped-call]
-                "sqs", queue_arn, self.Topic, self.Region, self.FilterPolicy, function  # type: ignore[attr-defined, attr-defined, attr-defined]
+            queue_policy = self._inject_sqs_queue_policy(self.Topic, queue_arn, queue_url, function)  # type: ignore[no-untyped-call]
+            subscription = self._inject_subscription(
+                "sqs", queue_arn, self.Topic, self.Region, self.FilterPolicy, self.RedrivePolicy, function
             )
             event_source = self._inject_sqs_event_source_mapping(function, role, queue_arn)  # type: ignore[no-untyped-call]
 
@@ -500,19 +510,24 @@ class SNS(PushEventSource):
 
         # SNS -> SQS(Existing) -> Lambda
         resources = []
-        queue_arn = self.SqsSubscription.get("QueueArn", None)  # type: ignore[attr-defined]
-        queue_url = self.SqsSubscription.get("QueueUrl", None)  # type: ignore[attr-defined]
+        sqs_subscription: Dict[str, Any] = sam_expect(
+            self.SqsSubscription, self.relative_id, "SqsSubscription", is_sam_event=True
+        ).to_be_a_map()
+        queue_arn = sqs_subscription.get("QueueArn", None)
+        queue_url = sqs_subscription.get("QueueUrl", None)
         if not queue_arn or not queue_url:
             raise InvalidEventException(self.relative_id, "No QueueARN or QueueURL provided.")
 
-        queue_policy_logical_id = self.SqsSubscription.get("QueuePolicyLogicalId", None)  # type: ignore[attr-defined]
-        batch_size = self.SqsSubscription.get("BatchSize", None)  # type: ignore[attr-defined]
-        enabled = self.SqsSubscription.get("Enabled", None)  # type: ignore[attr-defined]
+        queue_policy_logical_id = sqs_subscription.get("QueuePolicyLogicalId", None)
+        batch_size = sqs_subscription.get("BatchSize", None)
+        enabled = sqs_subscription.get("Enabled", None)
 
         queue_policy = self._inject_sqs_queue_policy(  # type: ignore[no-untyped-call]
-            self.Topic, queue_arn, queue_url, function, queue_policy_logical_id  # type: ignore[attr-defined]
+            self.Topic, queue_arn, queue_url, function, queue_policy_logical_id
+        )
+        subscription = self._inject_subscription(
+            "sqs", queue_arn, self.Topic, self.Region, self.FilterPolicy, self.RedrivePolicy, function
         )
-        subscription = self._inject_subscription("sqs", queue_arn, self.Topic, self.Region, self.FilterPolicy, function)  # type: ignore[attr-defined, attr-defined, attr-defined, no-untyped-call]
         event_source = self._inject_sqs_event_source_mapping(function, role, queue_arn, batch_size, enabled)  # type: ignore[no-untyped-call]
 
         resources = resources + event_source
@@ -520,7 +535,16 @@ class SNS(PushEventSource):
         resources.append(subscription)
         return resources
 
-    def _inject_subscription(self, protocol, endpoint, topic, region, filterPolicy, function):  # type: ignore[no-untyped-def]
+    def _inject_subscription(
+        self,
+        protocol: str,
+        endpoint: str,
+        topic: str,
+        region: Optional[str],
+        filterPolicy: Optional[Dict[str, Any]],
+        redrivePolicy: Optional[Dict[str, Any]],
+        function: Any,
+    ) -> SNSSubscription:
         subscription = SNSSubscription(self.logical_id, attributes=function.get_passthrough_resource_attributes())
         subscription.Protocol = protocol
         subscription.Endpoint = endpoint
@@ -532,6 +556,9 @@ class SNS(PushEventSource):
         if filterPolicy is not None:
             subscription.FilterPolicy = filterPolicy
 
+        if redrivePolicy is not None:
+            subscription.RedrivePolicy = redrivePolicy
+
         return subscription
 
     def _inject_sqs_queue(self, function):  # type: ignore[no-untyped-def]
@@ -616,7 +643,7 @@ class Api(PushEventSource):
 
         return {"explicit_api": explicit_api, "explicit_api_stage": {"suffix": stage_suffix}}
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         """If the Api event source has a RestApi property, then simply return the Lambda Permission resource allowing
         API Gateway to call the function. If no RestApi is provided, then additionally inject the path, method, and the
@@ -699,9 +726,9 @@ class Api(PushEventSource):
         partition = ArnGenerator.get_partition_name()  # type: ignore[no-untyped-call]
         uri = _build_apigw_integration_uri(function, partition)  # type: ignore[no-untyped-call]
 
-        editor = SwaggerEditor(swagger_body)  # type: ignore[no-untyped-call]
+        editor = SwaggerEditor(swagger_body)
 
-        if editor.has_integration(self.Path, self.Method):  # type: ignore[attr-defined, no-untyped-call]
+        if editor.has_integration(self.Path, self.Method):  # type: ignore[attr-defined]
             # Cannot add the Lambda Integration, if it is already present
             raise InvalidEventException(
                 self.relative_id,
@@ -717,6 +744,7 @@ class Api(PushEventSource):
         editor.add_lambda_integration(self.Path, self.Method, uri, self.Auth, api.get("Auth"), condition=condition)  # type: ignore[attr-defined, attr-defined, no-untyped-call]
 
         if self.Auth:  # type: ignore[attr-defined]
+            sam_expect(self.Auth, self.relative_id, "Auth", is_sam_event=True).to_be_a_map()  # type: ignore[attr-defined]
             method_authorizer = self.Auth.get("Authorizer")  # type: ignore[attr-defined]
             api_auth = api.get("Auth")
             api_auth = intrinsics_resolver.resolve_parameter_refs(api_auth)
@@ -785,6 +813,7 @@ class Api(PushEventSource):
                     editor.add_custom_statements(resource_policy.get("CustomStatements"))  # type: ignore[no-untyped-call]
 
         if self.RequestModel:  # type: ignore[attr-defined]
+            sam_expect(self.RequestModel, self.relative_id, "RequestModel", is_sam_event=True).to_be_a_map()  # type: ignore[attr-defined]
             method_model = self.RequestModel.get("Model")  # type: ignore[attr-defined]
 
             if method_model:
@@ -934,7 +963,7 @@ class AlexaSkill(PushEventSource):
 
     property_types = {"SkillId": PropertyType(False, is_str())}
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         function = kwargs.get("function")
 
@@ -953,7 +982,7 @@ class IoTRule(PushEventSource):
 
     property_types = {"Sql": PropertyType(True, is_str()), "AwsIotSqlVersion": PropertyType(False, is_str())}
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         function = kwargs.get("function")
 
@@ -1016,7 +1045,7 @@ class Cognito(PushEventSource):
             self.relative_id, "Cognito events must reference a Cognito UserPool in the same template."
         )
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         function = kwargs.get("function")
 
@@ -1102,7 +1131,7 @@ class HttpApi(PushEventSource):
 
         return {"explicit_api": explicit_api}
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore[no-untyped-call]
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
         """If the Api event source has a RestApi property, then simply return the Lambda Permission resource allowing
         API Gateway to call the function. If no RestApi is provided, then additionally inject the path, method, and the
@@ -1148,7 +1177,7 @@ class HttpApi(PushEventSource):
         editor = None
         if resources_to_link["explicit_api"].get("DefinitionBody"):
             try:
-                editor = OpenApiEditor(resources_to_link["explicit_api"].get("DefinitionBody"))  # type: ignore[no-untyped-call]
+                editor = OpenApiEditor(resources_to_link["explicit_api"].get("DefinitionBody"))
             except InvalidDocumentException as e:
                 api_logical_id = self.ApiId.get("Ref") if isinstance(self.ApiId, dict) else self.ApiId  # type: ignore[attr-defined]
                 raise InvalidResourceException(api_logical_id, " ".join(ex.message for ex in e.causes))
@@ -1201,9 +1230,9 @@ class HttpApi(PushEventSource):
 
         uri = _build_apigw_integration_uri(function, "${AWS::Partition}")  # type: ignore[no-untyped-call]
 
-        editor = OpenApiEditor(open_api_body)  # type: ignore[no-untyped-call]
+        editor = OpenApiEditor(open_api_body)
 
-        if manage_swagger and editor.has_integration(self.Path, self.Method):  # type: ignore[attr-defined, no-untyped-call]
+        if manage_swagger and editor.has_integration(self.Path, self.Method):  # type: ignore[attr-defined]
             # Cannot add the Lambda Integration, if it is already present
             raise InvalidEventException(
                 self.relative_id,
