@@ -1,14 +1,18 @@
 import copy
 import re
-from typing import Any, Iterator, Optional
+from typing import Any, Dict, Optional
 
-from samtranslator.model.intrinsics import ref, make_conditional, is_intrinsic, is_intrinsic_no_value
+from samtranslator.model.apigatewayv2 import ApiGatewayV2Authorizer
+from samtranslator.model.intrinsics import ref, make_conditional, is_intrinsic
 from samtranslator.model.exceptions import InvalidDocumentException, InvalidTemplateException
+from samtranslator.open_api.base_editor import BaseEditor
 from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
+from samtranslator.utils.types import Intrinsicable
+from samtranslator.utils.utils import dict_deep_get, InvalidValueType
 import json
 
 
-class OpenApiEditor(object):
+class OpenApiEditor(BaseEditor):
     """
     Wrapper class capable of parsing and generating OpenApi JSON.  This implements OpenApi spec just enough that SAM
     cares about. It is built to handle "partial Swagger" ie. Swagger that is incomplete and won't
@@ -22,17 +26,13 @@ class OpenApiEditor(object):
     essential for generating the same logical ID.
     """
 
-    _X_APIGW_INTEGRATION = "x-amazon-apigateway-integration"
     _X_APIGW_TAG_VALUE = "x-amazon-apigateway-tag-value"
     _X_APIGW_CORS = "x-amazon-apigateway-cors"
     _X_APIGW_ENDPOINT_CONFIG = "x-amazon-apigateway-endpoint-configuration"
-    _SERVERS = "servers"
-    _CONDITIONAL_IF = "Fn::If"
-    _X_ANY_METHOD = "x-amazon-apigateway-any-method"
-    _ALL_HTTP_METHODS = ["OPTIONS", "GET", "HEAD", "POST", "PUT", "DELETE", "PATCH"]
     _DEFAULT_PATH = "$default"
+    _DEFAULT_OPENAPI_TITLE = ref("AWS::StackName")
 
-    def __init__(self, doc):  # type: ignore[no-untyped-def]
+    def __init__(self, doc: Optional[Dict[str, Any]]) -> None:
         """
         Initialize the class with a swagger dictionary. This class creates a copy of the Swagger and performs all
         modifications on this copy.
@@ -40,7 +40,7 @@ class OpenApiEditor(object):
         :param dict doc: OpenApi document as a dictionary
         :raises InvalidDocumentException: If the input OpenApi document does not meet the basic OpenApi requirements.
         """
-        if not OpenApiEditor.is_valid(doc):
+        if not doc or not OpenApiEditor.is_valid(doc):
             raise InvalidDocumentException(
                 [
                     InvalidTemplateException(
@@ -51,45 +51,13 @@ class OpenApiEditor(object):
 
         self._doc = copy.deepcopy(doc)
         self.paths = self._doc["paths"]
-        self.security_schemes = self._doc.get("components", Py27Dict()).get("securitySchemes", Py27Dict())
-        self.definitions = self._doc.get("definitions", Py27Dict())
-        self.tags = self._doc.get("tags", [])
-        self.info = self._doc.get("info", Py27Dict())
-
-    def get_conditional_contents(self, item):  # type: ignore[no-untyped-def]
-        """
-        Returns the contents of the given item.
-        If a conditional block has been used inside the item, returns a list of the content
-        inside the conditional (both the then and the else cases). Skips {'Ref': 'AWS::NoValue'} content.
-        If there's no conditional block, then returns an list with the single item in it.
-
-        :param dict item: item from which the contents will be extracted
-        :return: list of item content
-        """
-        contents = [item]
-        if isinstance(item, dict) and self._CONDITIONAL_IF in item:
-            contents = item[self._CONDITIONAL_IF][1:]
-            contents = [content for content in contents if not is_intrinsic_no_value(content)]
-        return contents
-
-    def has_path(self, path, method=None):  # type: ignore[no-untyped-def]
-        """
-        Returns True if this Swagger has the given path and optional method
-        For paths with conditionals, only returns true if both items (true case, and false case) have the method.
-
-        :param string path: Path name
-        :param string method: HTTP method
-        :return: True, if this path/method is present in the document
-        """
-        if path not in self.paths:
-            return False
-
-        method = self._normalize_method_name(method)  # type: ignore[no-untyped-call]
-        if method:
-            for path_item in self.get_conditional_contents(self.paths.get(path)):  # type: ignore[no-untyped-call]
-                if not path_item or method not in path_item:
-                    return False
-        return True
+        try:
+            self.security_schemes = dict_deep_get(self._doc, "components.securitySchemes") or Py27Dict()
+            self.definitions = dict_deep_get(self._doc, "definitions") or Py27Dict()
+            self.tags = dict_deep_get(self._doc, "tags") or []
+            self.info = dict_deep_get(self._doc, "info") or Py27Dict()
+        except InvalidValueType as ex:
+            raise InvalidDocumentException([InvalidTemplateException(f"Invalid OpenApi document: {str(ex)}")]) from ex
 
     def is_integration_function_logical_id_match(self, path_name, method_name, logical_id):  # type: ignore[no-untyped-def]
         """
@@ -102,18 +70,25 @@ class OpenApiEditor(object):
         :param method_name: name of the method
         :param logical_id: logical id to compare against
         """
-        if not self.has_integration(path_name, method_name):  # type: ignore[no-untyped-call]
+        if not self.has_integration(path_name, method_name):
             return False
-        method_name = self._normalize_method_name(method_name)  # type: ignore[no-untyped-call]
+        method_name = self._normalize_method_name(method_name)
 
-        for method_definition in self.iter_on_method_definitions_for_path_at_method(path_name, method_name, False):  # type: ignore[no-untyped-call]
+        for method_definition in self.iter_on_method_definitions_for_path_at_method(path_name, method_name, False):
             integration = method_definition.get(self._X_APIGW_INTEGRATION, Py27Dict())
-
+            if not isinstance(integration, dict):
+                raise InvalidDocumentException(
+                    [
+                        InvalidTemplateException(
+                            f"Value of '{self._X_APIGW_INTEGRATION}' must be a dictionary according to Swagger spec."
+                        )
+                    ]
+                )
             # Extract the integration uri out of a conditional if necessary
             uri = integration.get("uri")
             if not isinstance(uri, dict):
                 return False
-            for uri_content in self.get_conditional_contents(uri):  # type: ignore[no-untyped-call]
+            for uri_content in self.get_conditional_contents(uri):
                 arn = uri_content.get("Fn::Sub", "")
 
                 # Extract lambda integration (${LambdaName.Arn}) and split ".Arn" off from it
@@ -128,72 +103,6 @@ class OpenApiEditor(object):
 
         return True
 
-    def method_has_integration(self, method):  # type: ignore[no-untyped-def]
-        """
-        Returns true if the given method contains a valid method definition.
-        This uses the get_conditional_contents function to handle conditionals.
-
-        :param dict method: method dictionary
-        :return: true if method has one or multiple integrations
-        """
-        for method_definition in self.get_conditional_contents(method):  # type: ignore[no-untyped-call]
-            if self.method_definition_has_integration(method_definition):  # type: ignore[no-untyped-call]
-                return True
-        return False
-
-    def method_definition_has_integration(self, method_definition):  # type: ignore[no-untyped-def]
-        """
-        Checks a method definition to make sure it has an apigw integration
-
-        :param dict method_defintion: method definition dictionary
-        :return: True if an integration exists
-        """
-        if method_definition.get(self._X_APIGW_INTEGRATION):
-            return True
-        return False
-
-    def has_integration(self, path, method):  # type: ignore[no-untyped-def]
-        """
-        Checks if an API Gateway integration is already present at the given path/method.
-        For paths with conditionals, it only returns True if both items (true case, false case) have the integration
-
-        :param string path: Path name
-        :param string method: HTTP method
-        :return: True, if an API Gateway integration is already present
-        """
-        method = self._normalize_method_name(method)  # type: ignore[no-untyped-call]
-
-        if not self.has_path(path, method):  # type: ignore[no-untyped-call]
-            return False
-
-        for path_item in self.get_conditional_contents(self.paths.get(path)):  # type: ignore[no-untyped-call]
-            method_definition = path_item.get(method)
-            if not (isinstance(method_definition, dict) and self.method_has_integration(method_definition)):  # type: ignore[no-untyped-call]
-                return False
-        # Integration present and non-empty
-        return True
-
-    def add_path(self, path: str, method: Optional[str] = None) -> None:
-        """
-        Adds the path/method combination to the Swagger, if not already present
-
-        :param string path: Path name
-        :param string method: HTTP method
-        :raises InvalidDocumentException: If the value of `path` in Swagger is not a dictionary
-        """
-        method = self._normalize_method_name(method)  # type: ignore[no-untyped-call]
-
-        path_dict = self.paths.setdefault(path, Py27Dict())
-
-        if not isinstance(path_dict, dict):
-            # Either customers has provided us an invalid Swagger, or this class has messed it somehow
-            raise InvalidDocumentException(
-                [InvalidTemplateException(f"Value of '{path}' path must be a dictionary according to Swagger spec.")]
-            )
-
-        for path_item in self.get_conditional_contents(path_dict):  # type: ignore[no-untyped-call]
-            path_item.setdefault(method, Py27Dict())
-
     def add_lambda_integration(  # type: ignore[no-untyped-def]
         self, path, method, integration_uri, method_auth_config=None, api_auth_config=None, condition=None
     ):
@@ -205,8 +114,8 @@ class OpenApiEditor(object):
         :param string integration_uri: URI for the integration.
         """
 
-        method = self._normalize_method_name(method)  # type: ignore[no-untyped-call]
-        if self.has_integration(path, method):  # type: ignore[no-untyped-call]
+        method = self._normalize_method_name(method)
+        if self.has_integration(path, method):
             # Not throwing an error- we will add lambda integrations to existing swagger if not present
             return
 
@@ -217,7 +126,8 @@ class OpenApiEditor(object):
         if condition:
             integration_uri = make_conditional(condition, integration_uri)
 
-        for path_item in self.get_conditional_contents(self.paths.get(path)):  # type: ignore[no-untyped-call]
+        for path_item in self.get_conditional_contents(self.paths.get(path)):
+            BaseEditor.validate_path_item_is_dict(path_item, path)
             # create as Py27Dict and insert key one by one to preserve input order
             if path_item[method] is None:
                 path_item[method] = Py27Dict()
@@ -237,45 +147,6 @@ class OpenApiEditor(object):
             if condition:
                 path_item[method] = make_conditional(condition, path_item[method])
 
-    def make_path_conditional(self, path: str, condition: str) -> None:
-        """
-        Wrap entire API path definition in a CloudFormation if condition.
-        :param path: path name
-        :param condition: condition name
-        """
-        self.paths[path] = make_conditional(condition, self.paths[path])
-
-    def iter_on_path(self) -> Iterator[str]:
-        """
-        Yields all the paths available in the Swagger. As a caller, if you add new paths to Swagger while iterating,
-        they will not show up in this iterator
-
-        :yields string: Path name
-        """
-
-        for path, _ in self.paths.items():
-            yield path
-
-    def iter_on_method_definitions_for_path_at_method(  # type: ignore[no-untyped-def]
-        self, path_name, method_name, skip_methods_without_apigw_integration=True
-    ):
-        """
-        Yields all the method definitions for the path+method combinations if path and/or method have IF conditionals.
-        If there are no conditionals, will just yield the single method definition at the given path and method name.
-
-        :param path_name: path name
-        :param method_name: method name
-        :param skip_methods_without_apigw_integration: if True, skips method definitions without apigw integration
-        :yields dict: method definition
-        """
-        normalized_method_name = self._normalize_method_name(method_name)  # type: ignore[no-untyped-call]
-
-        for path_item in self.get_conditional_contents(self.paths.get(path_name)):  # type: ignore[no-untyped-call]
-            for method_definition in self.get_conditional_contents(path_item.get(normalized_method_name)):  # type: ignore[no-untyped-call]
-                if skip_methods_without_apigw_integration and not self.method_definition_has_integration(method_definition):  # type: ignore[no-untyped-call]
-                    continue
-                yield method_definition
-
     def iter_on_all_methods_for_path(self, path_name, skip_methods_without_apigw_integration=True):  # type: ignore[no-untyped-def]
         """
         Yields all the (method name, method definition) tuples for the path, including those inside conditionals.
@@ -284,12 +155,16 @@ class OpenApiEditor(object):
         :param skip_methods_without_apigw_integration: if True, skips method definitions without apigw integration
         :yields list of (method name, method definition) tuples
         """
-        for path_item in self.get_conditional_contents(self.paths.get(path_name)):  # type: ignore[no-untyped-call]
+        for path_item in self.get_conditional_contents(self.paths.get(path_name)):
+            BaseEditor.validate_path_item_is_dict(path_item, path_name)
             for method_name, method in path_item.items():
-                for method_definition in self.get_conditional_contents(method):  # type: ignore[no-untyped-call]
-                    if skip_methods_without_apigw_integration and not self.method_definition_has_integration(method_definition):  # type: ignore[no-untyped-call]
+                for method_definition in self.get_conditional_contents(method):
+                    BaseEditor.validate_method_definition_is_dict(method_definition, path_name, method_name)
+                    if skip_methods_without_apigw_integration and not self.method_definition_has_integration(
+                        method_definition
+                    ):
                         continue
-                    normalized_method_name = self._normalize_method_name(method_name)  # type: ignore[no-untyped-call]
+                    normalized_method_name = self._normalize_method_name(method_name)
                     yield normalized_method_name, method_definition
 
     def add_timeout_to_method(self, api, path, method_name, timeout):  # type: ignore[no-untyped-def]
@@ -301,7 +176,7 @@ class OpenApiEditor(object):
         :param string method_name: Method name
         :param int timeout: Timeout amount, in milliseconds
         """
-        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):  # type: ignore[no-untyped-call]
+        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):
             method_definition[self._X_APIGW_INTEGRATION]["timeoutInMillis"] = timeout
 
     def add_path_parameters_to_method(self, api, path, method_name, path_parameters):  # type: ignore[no-untyped-def]
@@ -313,16 +188,16 @@ class OpenApiEditor(object):
         :param string method_name: Method name
         :param list path_parameters: list of strings of path parameters
         """
-        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):  # type: ignore[no-untyped-call]
+        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):
             # create path parameter list
             # add it here if it doesn't exist, merge with existing otherwise.
-            method_definition.setdefault("parameters", [])
+            parameters = method_definition.setdefault("parameters", [])
             for param in path_parameters:
                 # find an existing parameter with this name if it exists
                 existing_parameter = next(
                     (
                         existing_parameter
-                        for existing_parameter in method_definition.get("parameters", [])
+                        for existing_parameter in parameters
                         if existing_parameter.get("name") == param
                     ),
                     None,
@@ -338,7 +213,7 @@ class OpenApiEditor(object):
                     parameter["name"] = param
                     parameter["in"] = "path"
                     parameter["required"] = True
-                    method_definition.get("parameters").append(parameter)
+                    parameters.append(parameter)
 
     def add_payload_format_version_to_method(self, api, path, method_name, payload_format_version="2.0"):  # type: ignore[no-untyped-def]
         """
@@ -349,10 +224,10 @@ class OpenApiEditor(object):
         :param string method_name: Method name
         :param string payload_format_version: payload format version sent to the integration
         """
-        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):  # type: ignore[no-untyped-call]
+        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):
             method_definition[self._X_APIGW_INTEGRATION]["payloadFormatVersion"] = payload_format_version
 
-    def add_authorizers_security_definitions(self, authorizers):  # type: ignore[no-untyped-def]
+    def add_authorizers_security_definitions(self, authorizers: Dict[str, ApiGatewayV2Authorizer]) -> None:
         """
         Add Authorizer definitions to the securityDefinitions part of Swagger.
 
@@ -363,7 +238,12 @@ class OpenApiEditor(object):
         for authorizer_name, authorizer in authorizers.items():
             self.security_schemes[authorizer_name] = authorizer.generate_openapi()
 
-    def set_path_default_authorizer(self, path, default_authorizer, authorizers, api_authorizers):  # type: ignore[no-untyped-def]
+    def set_path_default_authorizer(
+        self,
+        path: str,
+        default_authorizer: str,
+        authorizers: Dict[str, ApiGatewayV2Authorizer],
+    ) -> None:
         """
         Adds the default_authorizer to the security block for each method on this path unless an Authorizer
         was defined at the Function/Path/Method level. This is intended to be used to set the
@@ -373,16 +253,17 @@ class OpenApiEditor(object):
         :param string path: Path name
         :param string default_authorizer: Name of the authorizer to use as the default. Must be a key in the
             authorizers param.
-        :param list authorizers: List of Authorizer configurations defined on the related Api.
+        :param dict authorizers: Dict of Authorizer configurations defined on the related Api.
         """
-        for path_item in self.get_conditional_contents(self.paths.get(path)):  # type: ignore[no-untyped-call]
+        for path_item in self.get_conditional_contents(self.paths.get(path)):
+            BaseEditor.validate_path_item_is_dict(path_item, path)
             for method_name, method in path_item.items():
-                normalized_method_name = self._normalize_method_name(method_name)  # type: ignore[no-untyped-call]
+                normalized_method_name = self._normalize_method_name(method_name)
                 # Excluding parameters section
                 if normalized_method_name == "parameters":
                     continue
                 if normalized_method_name != "options":
-                    normalized_method_name = self._normalize_method_name(method_name)  # type: ignore[no-untyped-call]
+                    normalized_method_name = self._normalize_method_name(method_name)
                     # It is possible that the method could have two definitions in a Fn::If block.
                     if normalized_method_name not in path_item:
                         raise InvalidDocumentException(
@@ -392,28 +273,18 @@ class OpenApiEditor(object):
                                 )
                             ]
                         )
-                    for method_definition in self.get_conditional_contents(method):  # type: ignore[no-untyped-call]
-                        # check if there is any method_definition given by customer
-                        if not method_definition:
-                            raise InvalidDocumentException(
-                                [
-                                    InvalidTemplateException(
-                                        f"Invalid method definition ({normalized_method_name}) for path: {path}"
-                                    )
-                                ]
-                            )
+                    for method_definition in self.get_conditional_contents(method):
                         # If no integration given, then we don't need to process this definition (could be AWS::NoValue)
-                        if not self.method_definition_has_integration(method_definition):  # type: ignore[no-untyped-call]
+                        BaseEditor.validate_method_definition_is_dict(method_definition, path, method_name)
+                        if not self.method_definition_has_integration(method_definition):
                             continue
-                        existing_security = method_definition.get("security", [])
+                        existing_security = method_definition.get("security")
                         if existing_security:
                             continue
-                        authorizer_list = []
-                        if authorizers:
-                            authorizer_list.extend(authorizers.keys())
+
                         security_dict = {}
-                        security_dict[default_authorizer] = self._get_authorization_scopes(  # type: ignore[no-untyped-call]
-                            api_authorizers, default_authorizer
+                        security_dict[default_authorizer] = self._get_authorization_scopes(
+                            authorizers, default_authorizer
                         )
                         authorizer_security = [security_dict]
 
@@ -454,15 +325,19 @@ class OpenApiEditor(object):
         if authorization_scopes is None:
             authorization_scopes = []
 
-        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):  # type: ignore[no-untyped-call]
-            existing_security = method_definition.get("security", [])
+        for method_definition in self.iter_on_method_definitions_for_path_at_method(path, method_name):
 
             security_dict = {}  # type: ignore[var-annotated]
             security_dict[authorizer_name] = []
 
             # Neither the NONE nor the AWS_IAM built-in authorizers support authorization scopes.
             if authorizer_name not in ["NONE", "AWS_IAM"]:
-                method_authorization_scopes = authorizers[authorizer_name].get("AuthorizationScopes")
+                authorizer = authorizers.get(authorizer_name, Py27Dict())
+                if not isinstance(authorizer, dict):
+                    raise InvalidDocumentException(
+                        [InvalidTemplateException(f"Type of authorizer '{authorizer_name}' must be a dictionary")]
+                    )
+                method_authorization_scopes = authorizer.get("AuthorizationScopes")
                 if authorization_scopes:
                     method_authorization_scopes = authorization_scopes
                 if authorizers[authorizer_name] and method_authorization_scopes:
@@ -470,12 +345,17 @@ class OpenApiEditor(object):
 
             authorizer_security = [security_dict]
 
+            existing_security = method_definition.get("security", [])
+            if not isinstance(existing_security, list):
+                raise InvalidDocumentException(
+                    [InvalidTemplateException(f"Type of security for path {path} method {method_name} must be a list")]
+                )
             # This assumes there are no authorizers already configured in the existing security block
             security = existing_security + authorizer_security
             if security:
                 method_definition["security"] = security
 
-    def add_tags(self, tags):  # type: ignore[no-untyped-def]
+    def add_tags(self, tags: Dict[str, Intrinsicable[str]]) -> None:
         """
         Adds tags to the OpenApi definition using an ApiGateway extension for tag values.
 
@@ -503,7 +383,7 @@ class OpenApiEditor(object):
                 tag[self._X_APIGW_TAG_VALUE] = value
                 self.tags.append(tag)
 
-    def add_endpoint_config(self, disable_execute_api_endpoint):  # type: ignore[no-untyped-def]
+    def add_endpoint_config(self, disable_execute_api_endpoint: Optional[Intrinsicable[bool]]) -> None:
         """Add endpoint configuration to _X_APIGW_ENDPOINT_CONFIG header in open api definition
 
         Following this guide:
@@ -518,6 +398,14 @@ class OpenApiEditor(object):
 
         servers_configurations = self._doc.get(self._SERVERS, [Py27Dict()])
         for config in servers_configurations:
+            if not isinstance(config, dict):
+                raise InvalidDocumentException(
+                    [
+                        InvalidTemplateException(
+                            f"Value of '{self._SERVERS}' item must be a dictionary according to Swagger spec."
+                        )
+                    ]
+                )
             endpoint_configuration = config.get(self._X_APIGW_ENDPOINT_CONFIG, {})
             endpoint_configuration[DISABLE_EXECUTE_API_ENDPOINT] = disable_execute_api_endpoint
             config[self._X_APIGW_ENDPOINT_CONFIG] = endpoint_configuration
@@ -560,6 +448,14 @@ class OpenApiEditor(object):
         ALLOW_CREDENTIALS = "allowCredentials"
         cors_headers = [ALLOW_ORIGINS, ALLOW_HEADERS, ALLOW_METHODS, EXPOSE_HEADERS, MAX_AGE, ALLOW_CREDENTIALS]
         cors_configuration = self._doc.get(self._X_APIGW_CORS, {})
+        if not isinstance(cors_configuration, dict):
+            raise InvalidDocumentException(
+                [
+                    InvalidTemplateException(
+                        f"Value of '{self._X_APIGW_CORS}' must be a dictionary according to Swagger spec."
+                    )
+                ]
+            )
 
         # intrinsics will not work if cors configuration is defined in open api and as a property to the HttpApi
         if allow_origins and is_intrinsic(allow_origins):
@@ -587,7 +483,7 @@ class OpenApiEditor(object):
 
         self._doc[self._X_APIGW_CORS] = cors_configuration
 
-    def add_description(self, description):  # type: ignore[no-untyped-def]
+    def add_description(self, description: Intrinsicable[str]) -> None:
         """Add description in open api definition, if it is not already defined
 
         :param string description: Description of the API
@@ -596,13 +492,22 @@ class OpenApiEditor(object):
             return
         self.info["description"] = description
 
-    def has_api_gateway_cors(self):  # type: ignore[no-untyped-def]
+    def add_title(self, title: Intrinsicable[str]) -> None:
+        """Add title in open api definition, if it is not already defined
+
+        :param string description: Description of the API
+        """
+        if self.info.get("title") != OpenApiEditor._DEFAULT_OPENAPI_TITLE:
+            return
+        self.info["title"] = title
+
+    def has_api_gateway_cors(self) -> bool:
         if self._doc.get(self._X_APIGW_CORS):
             return True
         return False
 
     @property
-    def openapi(self):  # type: ignore[no-untyped-def]
+    def openapi(self) -> Dict[str, Any]:
         """
         Returns a **copy** of the OpenApi specification as a dictionary.
 
@@ -636,7 +541,7 @@ class OpenApiEditor(object):
         if bool(data) and isinstance(data, dict) and isinstance(data.get("paths"), dict):
             if bool(data.get("openapi")):
                 return OpenApiEditor.safe_compare_regex_with_string(
-                    OpenApiEditor.get_openapi_version_3_regex(), data["openapi"]
+                    OpenApiEditor._OPENAPI_VERSION_3_REGEX, data["openapi"]
                 )
         return False
 
@@ -652,53 +557,10 @@ class OpenApiEditor(object):
         skeleton["openapi"] = "3.0.1"
         skeleton["info"] = Py27Dict()
         skeleton["info"]["version"] = "1.0"
-        skeleton["info"]["title"] = ref("AWS::StackName")
+        skeleton["info"]["title"] = OpenApiEditor._DEFAULT_OPENAPI_TITLE
         skeleton["paths"] = Py27Dict()
         return skeleton
 
-    @staticmethod
-    def _get_authorization_scopes(authorizers, default_authorizer):  # type: ignore[no-untyped-def]
-        """
-        Returns auth scopes for an authorizer if present
-        :param authorizers: authorizer definitions
-        :param default_authorizer: name of the default authorizer
-        """
-        if authorizers is not None:
-            if (
-                authorizers[default_authorizer]
-                and authorizers[default_authorizer].get("AuthorizationScopes") is not None
-            ):
-                return authorizers[default_authorizer].get("AuthorizationScopes")
-        return []
-
-    @staticmethod
-    def _normalize_method_name(method):  # type: ignore[no-untyped-def]
-        """
-        Returns a lower case, normalized version of HTTP Method. It also know how to handle API Gateway specific methods
-        like "ANY"
-
-        NOTE: Always normalize before using the `method` value passed in as input
-
-        :param string method: Name of the HTTP Method
-        :return string: Normalized method name
-        """
-        if not method or not isinstance(method, str):
-            return method
-
-        method = method.lower()
-        if method == "any":
-            return OpenApiEditor._X_ANY_METHOD
-        return method
-
-    @staticmethod
-    def get_openapi_version_3_regex() -> str:
-        openapi_version_3_regex = r"\A3(\.\d)(\.\d)?$"
-        return openapi_version_3_regex
-
-    @staticmethod
-    def safe_compare_regex_with_string(regex: str, data: Any) -> bool:
-        return re.match(regex, str(data)) is not None
-
     @staticmethod
     def get_path_without_trailing_slash(path):  # type: ignore[no-untyped-def]
         sub = re.sub(r"{([a-zA-Z0-9._-]+|proxy\+)}", "*", path)
