@@ -1,5 +1,6 @@
 ﻿from typing import Dict, List
 
+from samtranslator.model.exceptions import ExceptionWithMessage
 from samtranslator.public.sdk.resource import SamResourceType
 from samtranslator.public.intrinsics import is_intrinsics
 from samtranslator.swagger.swagger import SwaggerEditor
@@ -36,6 +37,7 @@ class Globals(object):
             "AutoPublishAlias",
             "Layers",
             "DeploymentPreference",
+            "RolePath",
             "PermissionsBoundary",
             "ReservedConcurrentExecutions",
             "ProvisionedConcurrencyConfig",
@@ -44,6 +46,7 @@ class Globals(object):
             "FileSystemConfigs",
             "CodeSigningConfigArn",
             "Architectures",
+            "SnapStart",
             "EphemeralStorage",
             "FunctionUrlConfig",
         ],
@@ -84,16 +87,18 @@ class Globals(object):
         SamResourceType.SimpleTable.value: ["SSESpecification"],
     }
     # unreleased_properties *must be* part of supported_properties too
-    unreleased_properties: Dict[str, List[str]] = {}
+    unreleased_properties: Dict[str, List[str]] = {
+        SamResourceType.Function.value: [],
+    }
 
-    def __init__(self, template):
+    def __init__(self, template):  # type: ignore[no-untyped-def]
         """
         Constructs an instance of this object
 
         :param dict template: SAM template to be parsed
         """
         self.supported_resource_section_names = [
-            x.replace(self._RESOURCE_PREFIX, "") for x in self.supported_properties.keys()
+            x.replace(self._RESOURCE_PREFIX, "") for x in self.supported_properties
         ]
         # Sort the names for stability in list ordering
         self.supported_resource_section_names.sort()
@@ -101,9 +106,9 @@ class Globals(object):
         self.template_globals = {}
 
         if self._KEYWORD in template:
-            self.template_globals = self._parse(template[self._KEYWORD])
+            self.template_globals = self._parse(template[self._KEYWORD])  # type: ignore[no-untyped-call]
 
-    def merge(self, resource_type, resource_properties):
+    def merge(self, resource_type, resource_properties):  # type: ignore[no-untyped-def]
         """
         Adds global properties to the resource, if necessary. This method is a no-op if there are no global properties
         for this resource type
@@ -122,7 +127,7 @@ class Globals(object):
         return global_props.merge(resource_properties)
 
     @classmethod
-    def del_section(cls, template):
+    def del_section(cls, template):  # type: ignore[no-untyped-def]
         """
         Helper method to delete the Globals section altogether from the template
 
@@ -134,7 +139,7 @@ class Globals(object):
             del template[cls._KEYWORD]
 
     @classmethod
-    def fix_openapi_definitions(cls, template):
+    def fix_openapi_definitions(cls, template):  # type: ignore[no-untyped-def]
         """
         Helper method to postprocess the resources to make sure the swagger doc version matches
         the one specified on the resource with flag OpenApiVersion.
@@ -158,7 +163,7 @@ class Globals(object):
                     (cls._OPENAPIVERSION in properties)
                     and (cls._MANAGE_SWAGGER in properties)
                     and SwaggerEditor.safe_compare_regex_with_string(
-                        SwaggerEditor.get_openapi_version_3_regex(), properties[cls._OPENAPIVERSION]
+                        SwaggerEditor._OPENAPI_VERSION_3_REGEX, properties[cls._OPENAPIVERSION]
                     )
                 ):
                     if not isinstance(properties[cls._OPENAPIVERSION], str):
@@ -170,7 +175,7 @@ class Globals(object):
                         if definition_body.get("swagger"):
                             del definition_body["swagger"]
 
-    def _parse(self, globals_dict):
+    def _parse(self, globals_dict):  # type: ignore[no-untyped-def]
         """
         Takes a SAM template as input and parses the Globals section
 
@@ -181,13 +186,13 @@ class Globals(object):
 
         globals = {}
         if not isinstance(globals_dict, dict):
-            raise InvalidGlobalsSectionException(self._KEYWORD, "It must be a non-empty dictionary")
+            raise InvalidGlobalsSectionException(self._KEYWORD, "It must be a non-empty dictionary")  # type: ignore[no-untyped-call]
 
         for section_name, properties in globals_dict.items():
-            resource_type = self._make_resource_type(section_name)
+            resource_type = self._make_resource_type(section_name)  # type: ignore[no-untyped-call]
 
             if resource_type not in self.supported_properties:
-                raise InvalidGlobalsSectionException(
+                raise InvalidGlobalsSectionException(  # type: ignore[no-untyped-call]
                     self._KEYWORD,
                     "'{section}' is not supported. "
                     "Must be one of the following values - {supported}".format(
@@ -196,15 +201,15 @@ class Globals(object):
                 )
 
             if not isinstance(properties, dict):
-                raise InvalidGlobalsSectionException(self._KEYWORD, "Value of ${section} must be a dictionary")
+                raise InvalidGlobalsSectionException(self._KEYWORD, "Value of ${section} must be a dictionary")  # type: ignore[no-untyped-call]
 
             supported = self.supported_properties[resource_type]
             supported_displayed = [
                 prop for prop in supported if prop not in self.unreleased_properties.get(resource_type, [])
             ]
-            for key, value in properties.items():
+            for key, _ in properties.items():
                 if key not in supported:
-                    raise InvalidGlobalsSectionException(
+                    raise InvalidGlobalsSectionException(  # type: ignore[no-untyped-call]
                         self._KEYWORD,
                         "'{key}' is not a supported property of '{section}'. "
                         "Must be one of the following values - {supported}".format(
@@ -213,11 +218,11 @@ class Globals(object):
                     )
 
             # Store all Global properties in a map with key being the AWS::Serverless::* resource type
-            globals[resource_type] = GlobalProperties(properties)
+            globals[resource_type] = GlobalProperties(properties)  # type: ignore[no-untyped-call]
 
         return globals
 
-    def _make_resource_type(self, key):
+    def _make_resource_type(self, key):  # type: ignore[no-untyped-def]
         return self._RESOURCE_PREFIX + key
 
 
@@ -341,18 +346,18 @@ class GlobalProperties(object):
 
     """
 
-    def __init__(self, global_properties):
+    def __init__(self, global_properties):  # type: ignore[no-untyped-def]
         self.global_properties = global_properties
 
-    def merge(self, local_properties):
+    def merge(self, local_properties):  # type: ignore[no-untyped-def]
         """
         Merge Global & local level properties according to the above rules
 
         :return local_properties: Dictionary of local properties
         """
-        return self._do_merge(self.global_properties, local_properties)
+        return self._do_merge(self.global_properties, local_properties)  # type: ignore[no-untyped-call]
 
-    def _do_merge(self, global_value, local_value):
+    def _do_merge(self, global_value, local_value):  # type: ignore[no-untyped-def]
         """
         Actually perform the merge operation for the given inputs. This method is used as part of the recursion.
         Therefore input values can be of any type. So is the output.
@@ -362,28 +367,25 @@ class GlobalProperties(object):
         :return: Merged result
         """
 
-        token_global = self._token_of(global_value)
-        token_local = self._token_of(local_value)
+        token_global = self._token_of(global_value)  # type: ignore[no-untyped-call]
+        token_local = self._token_of(local_value)  # type: ignore[no-untyped-call]
 
         # The following statements codify the rules explained in the doctring above
         if token_global != token_local:
-            return self._prefer_local(global_value, local_value)
+            return self._prefer_local(global_value, local_value)  # type: ignore[no-untyped-call]
 
-        elif self.TOKEN.PRIMITIVE == token_global == token_local:
-            return self._prefer_local(global_value, local_value)
+        if self.TOKEN.PRIMITIVE == token_global == token_local:
+            return self._prefer_local(global_value, local_value)  # type: ignore[no-untyped-call]
 
-        elif self.TOKEN.DICT == token_global == token_local:
-            return self._merge_dict(global_value, local_value)
+        if self.TOKEN.DICT == token_global == token_local:
+            return self._merge_dict(global_value, local_value)  # type: ignore[no-untyped-call]
 
-        elif self.TOKEN.LIST == token_global == token_local:
-            return self._merge_lists(global_value, local_value)
+        if self.TOKEN.LIST == token_global == token_local:
+            return self._merge_lists(global_value, local_value)  # type: ignore[no-untyped-call]
 
-        else:
-            raise TypeError(
-                "Unsupported type of objects. GlobalType={}, LocalType={}".format(token_global, token_local)
-            )
+        raise TypeError("Unsupported type of objects. GlobalType={}, LocalType={}".format(token_global, token_local))
 
-    def _merge_lists(self, global_list, local_list):
+    def _merge_lists(self, global_list, local_list):  # type: ignore[no-untyped-def]
         """
         Merges the global list with the local list. List merging is simply a concatenation = global + local
 
@@ -394,7 +396,7 @@ class GlobalProperties(object):
 
         return global_list + local_list
 
-    def _merge_dict(self, global_dict, local_dict):
+    def _merge_dict(self, global_dict, local_dict):  # type: ignore[no-untyped-def]
         """
         Merges the two dictionaries together
 
@@ -410,7 +412,7 @@ class GlobalProperties(object):
 
             if key in global_dict:
                 # Both local & global contains the same key. Let's do a merge.
-                global_dict[key] = self._do_merge(global_dict[key], local_dict[key])
+                global_dict[key] = self._do_merge(global_dict[key], local_dict[key])  # type: ignore[no-untyped-call]
 
             else:
                 # Key is not in globals, just in local. Copy it over
@@ -418,7 +420,7 @@ class GlobalProperties(object):
 
         return global_dict
 
-    def _prefer_local(self, global_value, local_value):
+    def _prefer_local(self, global_value, local_value):  # type: ignore[no-untyped-def]
         """
         Literally returns the local value whatever it may be. This method is useful to provide a unified implementation
         for cases that don't require special handling.
@@ -429,7 +431,7 @@ class GlobalProperties(object):
         """
         return local_value
 
-    def _token_of(self, input):
+    def _token_of(self, input):  # type: ignore[no-untyped-def]
         """
         Returns the token type of the input.
 
@@ -444,14 +446,12 @@ class GlobalProperties(object):
                 # Intrinsic functions are handled *exactly* like a primitive type because
                 # they resolve to a primitive type when creating a stack with CloudFormation
                 return self.TOKEN.PRIMITIVE
-            else:
-                return self.TOKEN.DICT
+            return self.TOKEN.DICT
 
-        elif isinstance(input, list):
+        if isinstance(input, list):
             return self.TOKEN.LIST
 
-        else:
-            return self.TOKEN.PRIMITIVE
+        return self.TOKEN.PRIMITIVE
 
     class TOKEN:
         """
@@ -463,17 +463,17 @@ class GlobalProperties(object):
         LIST = "list"
 
 
-class InvalidGlobalsSectionException(Exception):
-    """Exception raised when a Globals section is is invalid.
+class InvalidGlobalsSectionException(ExceptionWithMessage):
+    """Exception raised when a Globals section is invalid.
 
     Attributes:
         message -- explanation of the error
     """
 
-    def __init__(self, logical_id, message):
+    def __init__(self, logical_id, message):  # type: ignore[no-untyped-def]
         self._logical_id = logical_id
         self._message = message
 
     @property
-    def message(self):
+    def message(self):  # type: ignore[no-untyped-def]
         return "'{}' section is invalid. {}".format(self._logical_id, self._message)
